Index: trunk/kernel/syscalls/sys_mkfifo.c
===================================================================
--- trunk/kernel/syscalls/sys_mkfifo.c	(revision 1)
+++ trunk/kernel/syscalls/sys_mkfifo.c	(revision 23)
@@ -1,15 +1,16 @@
 /*
- * kern/sys_mkfifo.c - creates a FIFO named file
+ * sys_mkfifo.c - creates a named FIFO file.
  * 
- * Copyright (c) 2008,2009,2010,2011,2012 Ghassan Almaless
- * Copyright (c) 2011,2012 UPMC Sorbonne Universites
+ * Author    Alain Greiner (2016,2017)
  *
- * This file is part of ALMOS-kernel.
+ * Copyright (c) UPMC Sorbonne Universites
  *
  * ALMOS-kernel is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
  * the Free Software Foundation; version 2.0 of the License.
  *
- * ALMOS-kernel is distributed in the hope that it will be useful, but
+ * ALMOS-MKH is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
@@ -17,37 +18,83 @@
  *
  * You should have received a copy of the GNU General Public License
- * along with ALMOS-kernel; if not, write to the Free Software Foundation,
+ * along with ALMOS-MKH; if not, write to the Free Software Foundation,
  * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#include <kdmsg.h>
+#include <hal_types.h>
+#include <hal_uspace.h>
+#include <printk.h>
 #include <vfs.h>
-#include <sys-vfs.h>
-#include <task.h>
+#include <process.h>
 #include <thread.h>
 
-int sys_mkfifo (char *pathname, uint_t mode)
+////////////////////////////////////
+int sys_mkfifo ( char    * pathname,
+                 uint32_t  mode )
 {
-	register error_t err = 0;
-	struct task_s *task = current_task;
-	struct ku_obj ku_path;
+    error_t        error;
+    uint32_t       length;                           // pathname length (bytes)
+    char           kbuf[CONFIG_VFS_MAX_PATH_LENGTH];
 
-	current_thread->info.errno = ENOSYS;
-	return -1;
+    thread_t  * this    = CURRENT_THREAD;
+	process_t * process = this->process;
 
-	KU_BUFF(ku_path, pathname);
-	rwlock_rdlock(&task->cwd_lock);
-	if((err = vfs_mkfifo(&task->vfs_cwd, &ku_path, mode)))
+	if( pathname == NULL )
 	{
-		printk(INFO, "INFO: sys_mkfifo: Thread %x, CPU %d, Error Code %d\n", 
-		       current_thread, 
-		       cpu_get_id(), 
-		       err);
+        printk("\n[ERROR] in %s : pathname is NULL\n", __FUNCTION__ );
+		this->errno = EINVAL;
+		return -1;
+	}	
 
-		rwlock_unlock(&task->cwd_lock);
+    // check fd_array not full
+	if( process_fd_array_full() )
+	{
+        printk("\n[ERROR] in %s : file descriptor array full for process %x\n",
+               __FUNCTION__ , process->pid );
+		this->errno = ENFILE;
+        return -1;
+	}
+   
+    // get pathname length
+    length = hal_strlen_from_uspace( pathname );
+
+    if( length >= CONFIG_VFS_MAX_PATH_LENGTH )
+    {
+        printk("\n[ERROR] in %s : pathname too long\n", __FUNCTION__ );
+		this->errno = ENFILE;
+        return -1;
+    }
+ 
+	// get pathname copy in kernel space
+    hal_copy_from_uspace( kbuf, pathname, length );
+
+    // get cluster and local pointer on reference process
+    xptr_t      ref_xp  = process->ref_xp;
+    process_t * ref_ptr = (process_t *)GET_PTR( ref_xp );
+    cxy_t       ref_cxy = GET_CXY( ref_xp );
+
+    // get extended pointer on cwd inode
+    xptr_t cwd_xp = hal_remote_lwd( XPTR( ref_cxy , &ref_ptr->vfs_cwd_xp ) );
+   
+    // get the cwd lock in read mode from reference process
+	remote_rwlock_rd_lock( XPTR( ref_cxy , &ref_ptr->cwd_lock ) );
+
+    // call the relevant VFS function
+	error = vfs_mkfifo( cwd_xp,
+                        kbuf,
+                        mode );
+
+    // release the cwd lock
+	remote_rwlock_rd_unlock( XPTR( ref_cxy , &ref_ptr->cwd_lock ) );
+
+    if( error )
+	{
+        printk("\n[ERROR] in %s : cannot create named FIFO %s\n",
+               __FUNCTION__ , kbuf );
+		this->errno = error;
 		return -1;
 	}
-	rwlock_unlock(&task->cwd_lock);
    
 	return 0;
-}
+
+} // end sys_mkfifo()
@@ -1,15 +1,16 @@
  *
- * ALMOS-kernel is free software; you can redistribute it and/or modify it
+ * This file is part of ALMOS-MKH.
+ *
+ * ALMOS-MKH is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
  * the Free Software Foundation; version 2.0 of the License.
